@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2014 GraphicsMagick Group
+% Copyright (C) 2003 - 2019 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -36,6 +36,8 @@
 /*
   Include declarations.
 */
+#include <sys/types.h>
+#include <sys/utime.h>
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/utility.h"
@@ -52,10 +54,12 @@
 static char
   *NTslsearchpath = (char *) NULL;
 #endif
+#if defined(HasGS)
 static void
   *gs_dll_handle = (void *)NULL;
 static GhostscriptVectors
     gs_vectors;
+#endif /* if defined(HasGS) */
 
 static MagickPassFail NTstrerror_r(LONG errnum, char *strerrbuf, size_t  buflen);
 
@@ -66,6 +70,19 @@ static MagickPassFail NTstrerror_r(LONG errnum, char *strerrbuf, size_t  buflen)
 extern "C" BOOL WINAPI
   DllMain(HINSTANCE hinstDLL,DWORD fdwReason,LPVOID lpvReserved);
 #endif
+
+static const struct
+  {
+    const HKEY hkey;
+    const char *name;
+  }
+hkeys[2] =
+    {
+      { HKEY_CURRENT_USER,  "HKEY_CURRENT_USER"  },
+      { HKEY_LOCAL_MACHINE, "HKEY_LOCAL_MACHINE" }
+    };
+
+
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -123,7 +140,7 @@ MagickExport int NTclosedir(DIR *entry)
 %  A description of each parameter follows:
 %
 %    o hinstDLL: handle to the DLL module
-%   
+%
 %    o fdwReason: reason for calling function.
 %
 %          May have values:
@@ -156,9 +173,9 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL,DWORD fdwReason,LPVOID lpvReserved)
         char
           dll_path[DLL_PATH_MAX],
           current_path[ENV_VAR_MAX];
-        
+
         long count;
-        
+
         count = GetModuleFileName(hinstDLL,dll_path,DLL_PATH_MAX);
         if (count) {
           for ( ; count>0 ; --count)
@@ -262,6 +279,89 @@ MagickExport long MagickGetMMUPageSize()
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k G e t F i l e A t t r i b u t e s                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickGetFileAttributes() returns the file attributes for a specified
+%  file in a structure of type MagickStatStruct_t.
+%
+%  The format of the MagickGetFileAttributes method is:
+%
+%      int MagickGetFileAttributes(const char *filename,
+%                                  MagickStatStruct_t *statbuf)
+%
+%  A description of each parameter follows:
+%
+%    o filename:  Path to the file
+%
+%    o statbuf: A structure of type MagickStatStruct_t to populate.
+%
+%
+*/
+MagickExport int MagickGetFileAttributes(const char *filename,
+                                         MagickStatStruct_t *statbuf)
+{
+  if (MagickStat(filename, statbuf) != 0)
+    return -1;
+
+  return 0;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k S e t F i l e A t t r i b u t e s                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickSetFileAttributes() sets the access and modification time file
+%  attributes based on values provided via in a structure of type
+%  MagickStatStruct_t.
+%
+%  The format of the MagickGetFileAttributes method is:
+%
+%      int MagickSetFileAttributes(const char *filename,
+%                                  const MagickStatStruct_t *statbuf)
+%
+%  A description of each parameter follows:
+%
+%    o filename:  Path to the file
+%
+%    o statbuf: A structure of type MagickStatStruct_t to populate.
+%
+%
+*/
+MagickExport int MagickSetFileAttributes(const char *filename,
+                                         const MagickStatStruct_t *statbuf)
+{
+  /*
+    Setting file timestamps on Windows is actually almost the same as on POSIX systems.
+    https://msdn.microsoft.com/en-us/library/4wacf567.aspx
+   */
+  struct _utimbuf
+    ut;
+
+  ut.actime = statbuf->st_atime;
+  ut.modtime = statbuf->st_mtime;
+
+  if (_utime(filename, &ut) == -1)
+    return -1;
+
+  return 0;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   I s W i n d o w s 9 5                                                     %
 %                                                                             %
 %                                                                             %
@@ -355,7 +455,7 @@ MagickUnhandledExceptionFilter(struct _EXCEPTION_POINTERS *info)
     ULONG_PTR                ExceptionInformation[EXCEPTION_MAXIMUM_PARAMETERS];
     } EXCEPTION_RECORD, *PEXCEPTION_RECORD;
 
-    https://msdn.microsoft.com/en-us/library/windows/desktop/ms679284%28v=vs.85%29.aspx 
+    https://msdn.microsoft.com/en-us/library/windows/desktop/ms679284%28v=vs.85%29.aspx
     We don't care about PCONTEXT because it is processor-specific and includes low level
     details such as register contents.
 
@@ -522,7 +622,8 @@ MagickUnhandledExceptionFilter(struct _EXCEPTION_POINTERS *info)
       FormatString(message,
                    "%s: caught exception 0x%08X \"%s\"...",
                    GetClientName(), (unsigned int) code, text);
-      if (write(STDERR_FILENO,message,strlen(message)) == -1)
+      if (write(STDERR_FILENO,message,
+                (MAGICK_POSIX_IO_SIZE_T) strlen(message)) == -1)
         {
           /* Exists to quench warning */
         }
@@ -823,6 +924,7 @@ MagickExport void *NTdlsym(void *handle,const char *name)
   LPFNDLLFUNC1
     lpfnDllFunc1;
 
+  /* FARPROC GetProcAddress(HMODULE hModule,LPCSTR lpProcName); */
   lpfnDllFunc1=(LPFNDLLFUNC1) GetProcAddress(handle,name);
   if (!lpfnDllFunc1)
     return((void *) NULL);
@@ -889,21 +991,41 @@ MagickExport int NTmunmap(void *map,size_t length)
 */
 MagickExport double NTElapsedTime(void)
 {
-  union
-  {
-    FILETIME
-      filetime;
+  /*
+    See
+    https://msdn.microsoft.com/en-us/library/windows/desktop/ms644905%28v=vs.85%29.aspx
+    https://msdn.microsoft.com/en-us/library/windows/desktop/ms644904%28v=vs.85%29.aspx
+  */
+  static LARGE_INTEGER pFrequency = {0};
+  LARGE_INTEGER CurrentTime;
 
-    __int64
-      filetime64;
-  } elapsed_time;
+  if (pFrequency.QuadPart == 0)
+    if (QueryPerformanceFrequency(&pFrequency) == 0)
+      pFrequency.QuadPart = 1;  /* Mark that the Performance counter is NOT supported. */
 
-  SYSTEMTIME
-    system_time;
+  if (pFrequency.QuadPart > 1)
+    {
+      QueryPerformanceCounter(&CurrentTime);
+      return (double) CurrentTime.QuadPart / pFrequency.QuadPart;
+    }
+  else
+    {
+      union
+      {
+        FILETIME
+        filetime;
 
-  GetSystemTime(&system_time);
-  SystemTimeToFileTime(&system_time,&elapsed_time.filetime);
-  return((double) 1.0e-7*elapsed_time.filetime64);
+        __int64
+        filetime64;
+      } elapsed_time;
+
+      SYSTEMTIME
+        system_time;
+
+      GetSystemTime(&system_time);
+      SystemTimeToFileTime(&system_time,&elapsed_time.filetime);
+      return((double) 1.0e-7*elapsed_time.filetime64);
+    }
 }
 
 /*
@@ -1069,7 +1191,7 @@ char *NTGetLastError(void)
  */
 static int
 NTGetRegistryValue(HKEY hkeyroot, const char *key, const char *name,
-		   char *ptr, int *plen)
+                   char *ptr, int *plen)
 {
   HKEY
     hkey;
@@ -1091,22 +1213,22 @@ NTGetRegistryValue(HKEY hkeyroot, const char *key, const char *name,
       keytype = REG_SZ;
       cbData = *plen;
       if (bptr == (BYTE *) NULL)
-	bptr = &b;  /* Registry API won't return ERROR_MORE_DATA */
+        bptr = &b;  /* Registry API won't return ERROR_MORE_DATA */
       /* if ptr is NULL */
       rc = RegQueryValueExA(hkey, (char *)name, 0, &keytype, bptr, &cbData);
       RegCloseKey(hkey);
       if (rc == ERROR_SUCCESS)
-	{
-	  *plen = cbData;
-	  return 0;  /* found environment variable and copied it */
-	}
+        {
+          *plen = cbData;
+          return 0;  /* found environment variable and copied it */
+        }
       else
-	if (rc == ERROR_MORE_DATA)
-	  {
-	    /* buffer wasn't large enough */
-	    *plen = cbData;
-	    return -1;
-	  }
+        if (rc == ERROR_MORE_DATA)
+          {
+            /* buffer wasn't large enough */
+            *plen = cbData;
+            return -1;
+          }
     }
   return 1;  /* not found */
 }
@@ -1117,8 +1239,8 @@ NTGetRegistryValue(HKEY hkeyroot, const char *key, const char *name,
 */
 static MagickPassFail
 NTGhostscriptFind(const char **gs_productfamily,
-		  int *gs_major_version,
-		  int *gs_minor_version)
+                  int *gs_major_version,
+                  int *gs_minor_version)
 {
   /*
     These are the Ghostscript product versions we will search for.
@@ -1128,11 +1250,12 @@ NTGhostscriptFind(const char **gs_productfamily,
       "GPL Ghostscript",
       "GNU Ghostscript",
       "AFPL Ghostscript",
-      "Aladdin Ghostscript" 
+      "Aladdin Ghostscript"
     };
 
   unsigned int
-    product_index;
+    product_index,
+    whence;
 
   MagickPassFail
     status;
@@ -1141,135 +1264,137 @@ NTGhostscriptFind(const char **gs_productfamily,
   *gs_productfamily=NULL;
 
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-			"Searching for Ghostscript...");
+                        "Searching for Ghostscript...");
 
   /* Minimum version of Ghostscript is 5.50 */
   *gs_major_version=5;
   *gs_minor_version=49;
-  for (product_index=0; product_index < sizeof(products)/sizeof(products[0]);
-       ++product_index)
-    {
-      HKEY
-	hkey,
-	hkeyroot;
+  for(whence=0; whence<=1; whence++)
+  {
+    const HKEY hkeyroot = hkeys[whence].hkey;
+    const char *KeyRootStr = hkeys[whence].name;
 
-      LONG
-	winstatus;
+    for(product_index=0; product_index < sizeof(products)/sizeof(products[0]); ++product_index)
+    {
+      HKEY hkey;
+      LONG winstatus;
 
       REGSAM
-	open_key_mode;
+        open_key_mode;
 
       char
-	key[MaxTextExtent],
-	last_error_msg[MaxTextExtent];
+        key[MaxTextExtent],
+        last_error_msg[MaxTextExtent];
 
       (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-			    "  Searching for %s...",
-			    products[product_index]);
+                            "  Searching for %s...",
+                            products[product_index]);
       FormatString(key,"SOFTWARE\\%s",products[product_index]);
-      hkeyroot = HKEY_LOCAL_MACHINE;
+
       /*
-	long WINAPI RegOpenKeyEx(const HKEY hKey, const LPCTSTR
-	lpSubKey, const DWORD ulOptions, const REGSAM samDesired,
-	PHKEY phkResult)
+        long WINAPI RegOpenKeyEx(const HKEY hKey, const LPCTSTR
+        lpSubKey, const DWORD ulOptions, const REGSAM samDesired,
+        PHKEY phkResult)
       */
       open_key_mode=KEY_READ;
       if ((winstatus=RegOpenKeyExA(hkeyroot, key, 0, open_key_mode, &hkey))
-	  == ERROR_SUCCESS)
-	{
-	  DWORD
-	    cbData;
+          == ERROR_SUCCESS)
+        {
+          DWORD
+            cbData;
 
-	  int
-	    n;
+          int
+            n;
 
-	  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-				"    RegOpenKeyExA() opened "
-				"\"HKEY_LOCAL_MACHINE\\%s\"",
-				key);
-	  /* Now enumerate the keys */
-	  cbData = sizeof(key) / sizeof(char);
-	  n=0;
-	  /*
-	    LONG WINAPI RegEnumKeyEx(HKEY hKey, DWORD dwIndex, LPTSTR
-	    lpName, LPDWORD lpcName, LPDWORD lpReserved, LPTSTR
-	    lpClass, LPDWORD lpcClass, PFILETIME lpftLastWriteTime)
+          (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                                "    RegOpenKeyExA() opened "
+                                "\"%s\\%s\"",
+                                KeyRootStr, key);
+          /* Now enumerate the keys */
+          cbData = sizeof(key) / sizeof(char);
+          n=0;
+          /*
+            LONG WINAPI RegEnumKeyEx(HKEY hKey, DWORD dwIndex, LPTSTR
+            lpName, LPDWORD lpcName, LPDWORD lpReserved, LPTSTR
+            lpClass, LPDWORD lpcClass, PFILETIME lpftLastWriteTime)
 
-	    Enumerates the subkeys of the specified open registry key. 
+            Enumerates the subkeys of the specified open registry key.
 
-	    RegEnumKeyA is is provided only for compatibility with
-	    16-bit versions of Windows.
-	  */
-	  while ((winstatus=RegEnumKeyA(hkey, n, key, cbData)) == ERROR_SUCCESS)
-	    {
-	      int
-		major_version,
-		minor_version;
+            RegEnumKeyA is is provided only for compatibility with
+            16-bit versions of Windows.
+          */
+          while ((winstatus=RegEnumKeyA(hkey, n, key, cbData)) == ERROR_SUCCESS)
+            {
+              int
+                major_version,
+                minor_version;
 
-	      (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-				    "      RegEnumKeyA enumerated \"%s\"",key);
-	      n++;
-	      major_version=0;
-	      minor_version=0;
-	      if (sscanf(key,"%d.%d",&major_version,&minor_version) != 2)
-		continue;
+              (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                                    "      RegEnumKeyA enumerated \"%s\"",key);
+              n++;
+              major_version=0;
+              minor_version=0;
+              if (sscanf(key,"%d.%d",&major_version,&minor_version) != 2)
+                continue;
 
-	      (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-				    "      Found Ghostscript (%s) version %d.%02d",
-				    products[product_index],
-				    major_version,
-				    minor_version);
+              (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                                    "      Found Ghostscript (%s) version %d.%02d",
+                                    products[product_index],
+                                    major_version,
+                                    minor_version);
 
-	      if ((major_version > *gs_major_version) ||
-		  ((major_version == *gs_major_version) &&
-		   (minor_version > *gs_minor_version)))
-		{
-		  *gs_productfamily=products[product_index];
-		  *gs_major_version=major_version;
-		  *gs_minor_version=minor_version;
-		  status=MagickPass;
-		}
-	    }
-	  if (winstatus != ERROR_NO_MORE_ITEMS)
-	    {
-	      (void) NTstrerror_r(winstatus,last_error_msg,sizeof(last_error_msg));
-	      (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-				    "      RegEnumKeyA (%s)",
-				    last_error_msg);
-	    }
-	  /*
-	    LONG WINAPI RegCloseKey(HKEY hKey)
-	    
-	    Close the registry key.
-	  */
-	  winstatus=RegCloseKey(hkey);
-	}
+              if ((major_version > *gs_major_version) ||
+                  ((major_version == *gs_major_version) &&
+                   (minor_version > *gs_minor_version)))
+                {
+                  *gs_productfamily=products[product_index];
+                  *gs_major_version=major_version;
+                  *gs_minor_version=minor_version;
+                  status=MagickPass;
+                }
+            }
+          if (winstatus != ERROR_NO_MORE_ITEMS)
+            {
+              (void) NTstrerror_r(winstatus,last_error_msg,sizeof(last_error_msg));
+              (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                                    "      RegEnumKeyA (%s)",
+                                    last_error_msg);
+            }
+          /*
+            LONG WINAPI RegCloseKey(HKEY hKey)
+
+            Close the registry key.
+          */
+          winstatus=RegCloseKey(hkey);
+        }
       else
-	{
-	  /*
-	    If the function fails, the return value is a nonzero error
-	    code defined in Winerror.h. You can use the FormatMessage
-	    function with the FORMAT_MESSAGE_FROM_SYSTEM flag to get a
-	    generic description of the error.
-	   */
-	  (void) NTstrerror_r(winstatus,last_error_msg,sizeof(last_error_msg));
-	  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-				"    RegOpenKeyExA() failed to open "
-				"\"HKEY_LOCAL_MACHINE\\%s\" (%s)",
-				key,last_error_msg);
-	}
+        {
+          /*
+            If the function fails, the return value is a nonzero error
+            code defined in Winerror.h. You can use the FormatMessage
+            function with the FORMAT_MESSAGE_FROM_SYSTEM flag to get a
+            generic description of the error.
+           */
+          (void) NTstrerror_r(winstatus,last_error_msg,sizeof(last_error_msg));
+          (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                                "    RegOpenKeyExA() failed to open "
+                                "\"%s\\%s\" (%s)", KeyRootStr,
+                                key,last_error_msg);
+        }
     }
+  }
+
   if (status != MagickFail)
     {
       (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-			    "Selected Ghostscript (%s) version %d.%02d",
-			    *gs_productfamily,*gs_major_version,
-			    *gs_minor_version);
+                            "Selected Ghostscript (%s) version %d.%02d",
+                            *gs_productfamily,*gs_major_version,
+                            *gs_minor_version);
     }
   else
     {
       (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-			    "Failed to find Ghostscript!");
+                            "Failed to find Ghostscript!");
       *gs_major_version=0;
       *gs_minor_version=0;
     }
@@ -1291,23 +1416,12 @@ NTGhostscriptGetString(const char *name, char *ptr, const size_t len)
     gs_major_version=0,
     gs_minor_version=0;
 
-  struct
-  {
-    const HKEY hkey;
-    const char *name;
-  }
-  hkeys[2] =
-    {
-      { HKEY_CURRENT_USER,  "HKEY_CURRENT_USER"  },
-      { HKEY_LOCAL_MACHINE, "HKEY_LOCAL_MACHINE" }
-    };
-
   unsigned int
     i;
 
   int
     length;
-  
+
   char
     key[MaxTextExtent];
 
@@ -1315,34 +1429,35 @@ NTGhostscriptGetString(const char *name, char *ptr, const size_t len)
 
   if (NULL == gs_productfamily)
     (void) NTGhostscriptFind(&gs_productfamily,&gs_major_version,
-			     &gs_minor_version);
+                             &gs_minor_version);
 
   if (NULL == gs_productfamily)
     return MagickFail;
 
   FormatString(key,"SOFTWARE\\%s\\%d.%02d",gs_productfamily,
-	       gs_major_version, gs_minor_version);
-  
+               gs_major_version, gs_minor_version);
+
   for (i=0; i < sizeof(hkeys)/sizeof(hkeys[0]); ++i)
     {
       length = (int) len;
       if (NTGetRegistryValue(hkeys[i].hkey, key, name, ptr, &length) == 0)
-	{
-	  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-				"Registry: \"%s\\%s\\%s\"=\"%s\"",
-				hkeys[i].name,key,name,ptr);
-	  return MagickPass;
-	}
+        {
+          (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                                "Registry: \"%s\\%s\\%s\"=\"%s\"",
+                                hkeys[i].name,key,name,ptr);
+          return MagickPass;
+        }
       else
-	{
-	  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-				"Failed lookup: \"%s\\%s\\%s\"",
-				hkeys[i].name,key,name);
-	}
+        {
+          (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                                "Failed lookup: \"%s\\%s\\%s\"",
+                                hkeys[i].name,key,name);
+        }
     }
 
   return MagickFail;
 }
+#if defined(HasGS)
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1484,14 +1599,14 @@ MagickExport int NTGhostscriptEXE(char *path, int path_length)
       (void) strlcpy(cache,gsexe,sizeof(cache));
 
       if (NTGhostscriptDLL(cache,sizeof(cache)))
-	{
-	  p = strrchr(cache, '\\');
-	  if (p) {
-	    p++;
-	    *p = 0;
+        {
+          p = strrchr(cache, '\\');
+          if (p) {
+            p++;
+            *p = 0;
             (void) strlcat(cache,gsexe,sizeof(cache));
-	  }
-	}
+          }
+        }
       result=cache;
     }
 
@@ -1503,6 +1618,7 @@ MagickExport int NTGhostscriptEXE(char *path, int path_length)
 
   return FALSE;
 }
+#endif /* if defined(HasGS) */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1547,50 +1663,91 @@ MagickExport int NTGhostscriptFonts(char *path, int path_length)
     Ghostscript 7.0X does not include the Resource entry.
 
     Search path used by GPL Ghostscript 9.10 (2013-08-30):
-      C:\Program Files\gs\gs9.10\bin ; C:\Program Files\gs\gs9.10\lib ;
-      C:\Program Files\gs\gs9.10\fonts ; %rom%Resource/Init/ ; %rom%lib/ ;
-      c:/gs/gs9.10/Resource/Init ; c:/gs/gs9.10/lib ;
-      c:/gs/gs9.10/Resource/Font ; c:/gs/fonts
+    C:\Program Files\gs\gs9.10\bin ; C:\Program Files\gs\gs9.10\lib ;
+    C:\Program Files\gs\gs9.10\fonts ; %rom%Resource/Init/ ; %rom%lib/ ;
+    c:/gs/gs9.10/Resource/Init ; c:/gs/gs9.10/lib ;
+    c:/gs/gs9.10/Resource/Font ; c:/gs/fonts
 
   */
   {
+#define URW_FONT_HELVETICA_FILE "n019003l.pfb"
     const char
       *end = NULL,
       *start = gs_lib_path;
-        
-    end=start+strlen(start);
+
+    end = start+strlen(start);
     while ( start < end )
       {
         char
-          font_dir[MaxTextExtent],
           font_dir_file[MaxTextExtent];
-            
+
         const char
           *separator;
-            
+
         int
           length;
-            
+
         separator = strchr(start,DirectoryListSeparator);
         if (separator)
           length=separator-start;
         else
           length=end-start;
-        (void) strlcpy(font_dir,start,Min(length+1,MaxTextExtent));
-        (void) strlcpy(font_dir_file,font_dir,MaxTextExtent);
+        length = Min(length+1,MaxTextExtent);
+
+        (void) strlcpy(font_dir_file,start,length);
         (void) strlcat(font_dir_file,DirectorySeparator,MaxTextExtent);
-        (void) strlcat(font_dir_file,"fonts.dir",MaxTextExtent);
+        (void) strlcat(font_dir_file,URW_FONT_HELVETICA_FILE,MaxTextExtent);
         if (IsAccessible(font_dir_file))
           {
-            (void) strlcpy(path,font_dir,path_length);
+            (void) strlcpy(path, start, Min(length,path_length));
             (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
                                   "Ghostscript fonts in directory \"%s\"",
                                   path);
             return TRUE;
           }
-        start += length+1;
+        start += length;
+      }
+
+
+    // Second scan will try to get common fonts for all gs versions.
+    start = gs_lib_path;
+    end = start + strlen(start);
+    while ( start < end )
+      {
+        char
+          font_dir_file[MaxTextExtent];
+
+        const char
+          *separator, *gsdir;
+
+        int
+          length;
+
+        separator = strchr(start,DirectoryListSeparator);
+        gsdir = strstr(start,"/gs/gs");
+        if (gsdir==NULL) gsdir = strstr(start,"\\gs\\gs");
+        if (gsdir!=NULL && (separator==NULL || gsdir<separator))
+          {
+            length = Min((gsdir-start)+5,MaxTextExtent);
+            (void) strlcpy(font_dir_file,start,length);
+            (void) strlcat(font_dir_file,"fonts",MaxTextExtent);
+            (void) strlcat(font_dir_file,DirectorySeparator,MaxTextExtent);
+            (void) strlcat(font_dir_file,URW_FONT_HELVETICA_FILE,MaxTextExtent);
+            if (IsAccessible(font_dir_file))
+              {
+                (void) strlcpy(path,font_dir_file,Min(length+5,path_length));  // str size "fonts" is 5.
+                (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                                      "Ghostscript common fonts in directory \"%s\"",
+                                      path);
+                return TRUE;
+              }
+          }
+
+        if (separator==NULL) break;
+        start = separator + 1;
       }
   }
+
   {
     /*
       Check "c:/gs/fonts" since Ghostscript also looks there.  This
@@ -1603,15 +1760,15 @@ MagickExport int NTGhostscriptFonts(char *path, int path_length)
       different GS_LIB_DEFAULT (which includes AROOTDIR) definition.
     */
 
-    const char *
-      gs_font_dir           = "c:\\gs\\fonts";
+    const char
+      gs_font_dir[] = "c:\\gs\\fonts";
 
     char
       font_dir_file[MaxTextExtent];
 
     (void) strlcpy(font_dir_file,gs_font_dir,MaxTextExtent);
     (void) strlcat(font_dir_file,DirectorySeparator,MaxTextExtent);
-    (void) strlcat(font_dir_file,"fonts.dir",MaxTextExtent);
+    (void) strlcat(font_dir_file,URW_FONT_HELVETICA_FILE,MaxTextExtent);
     if (IsAccessible(font_dir_file))
       {
         (void) strlcpy(path,gs_font_dir,path_length);
@@ -1624,6 +1781,7 @@ MagickExport int NTGhostscriptFonts(char *path, int path_length)
 
   return FALSE;
 }
+#if defined(HasGS)
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1714,6 +1872,7 @@ MagickExport int NTGhostscriptUnLoadDLL(void)
 
   return False;
 }
+#endif /* if defined(HasGS) */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1797,7 +1956,7 @@ MagickExport unsigned char *NTRegistryKeyLookup(const char *subkey)
 
       LONG
         res;
-      
+
       FormatString(package_key,"SOFTWARE\\%s\\%s\\Q:%d", MagickPackageName,
                    MagickLibVersionText,QuantumDepth);
 
@@ -1817,29 +1976,32 @@ MagickExport unsigned char *NTRegistryKeyLookup(const char *subkey)
   {
     unsigned char
       *dest;
-    
+
     DWORD
       size,
       type;
 
     LONG
       res;
-    
+
     size = 32;
     dest = MagickAllocateMemory(unsigned char *,size);
-    
-    res = RegQueryValueExA (reg_key, subkey, 0, &type, dest, &size);
-    if (res == ERROR_MORE_DATA && type == REG_SZ)
+
+    if (dest != (unsigned char *) NULL)
       {
-        MagickReallocMemory(unsigned char *,dest,size);
         res = RegQueryValueExA (reg_key, subkey, 0, &type, dest, &size);
+        if (res == ERROR_MORE_DATA && type == REG_SZ)
+          {
+            MagickReallocMemory(unsigned char *,dest,size);
+            res = RegQueryValueExA (reg_key, subkey, 0, &type, dest, &size);
+          }
+
+        if (type != REG_SZ || res != ERROR_SUCCESS)
+          {
+            MagickFreeMemory(dest);
+          }
       }
-    
-    if (type != REG_SZ || res != ERROR_SUCCESS)
-      {
-        MagickFreeMemory(dest);
-      }
-    
+
     return dest;
   }
 }
@@ -1937,7 +2099,7 @@ MagickExport unsigned char *NTResourceToBlob(const char *id)
     {
       return((unsigned char *) NULL);
     }
-  blob=MagickAllocateMemory(unsigned char *,length+1);
+  blob=MagickAllocateMemory(unsigned char *,(size_t) length+1);
   if (blob != (unsigned char *) NULL)
     {
       (void) memcpy(blob,value,length);
@@ -1988,20 +2150,20 @@ NTstrerror_r(LONG errnum, char *strerrbuf, size_t  buflen)
   if (buflen > 0)
     strerrbuf[0]='\0';
   if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
-		    FORMAT_MESSAGE_FROM_SYSTEM,NULL,errnum,
-		    MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
-		    (LPTSTR) &buffer,0,NULL))
+                    FORMAT_MESSAGE_FROM_SYSTEM,NULL,errnum,
+                    MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
+                    (LPTSTR) &buffer,0,NULL))
     {
       if (strlcpy(strerrbuf,buffer,buflen) < buflen)
-	{
-	  size_t
-	    index;
+        {
+          size_t
+            index;
 
-	  for (index=0; strerrbuf[index] != 0; index++)
-	    if (strerrbuf[index] == '\015')
-	      strerrbuf[index]='\0';
-	  status=MagickPass;
-	}
+          for (index=0; strerrbuf[index] != 0; index++)
+            if (strerrbuf[index] == '\015')
+              strerrbuf[index]='\0';
+          status=MagickPass;
+        }
       LocalFree(buffer);
     }
   return status;
@@ -2269,11 +2431,11 @@ MagickExport int NTftruncate(int filedes, off_t length)
     The CreateFileMapping() function may also be used to extend a
     file's length. The filler byte values are not defined in the
     documentation.
-  */ 
+  */
   status=chsize(filedes,length);
 
   /*
-    It is not documented if _chsize preserves the seek 
+    It is not documented if _chsize preserves the seek
     position, so restore the seek position like ftruncate
     does
   */
